local AddonName, GGL = ...
--[[-----------------------------------------------------------------------------
Label Widget
Displays text and optionally an icon.
-------------------------------------------------------------------------------]]
local Type, Version = "GoldGoblinLootGUIWidget-ProgressBar", 1
local AceGUI = LibStub and LibStub("AceGUI-3.0", true)
if not AceGUI or (AceGUI:GetWidgetVersion(Type) or 0) >= Version then return end

-- Lua APIs
local max, pairs = math.max, pairs
local tonumber = tonumber
local format = string.format

-- WoW APIs
local CreateFrame, UIParent = CreateFrame, UIParent
local GetItemIcon, GetItemInfo = GetItemIcon, GetItemInfo

--[[-----------------------------------------------------------------------------
Support functions
-------------------------------------------------------------------------------]]


--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
    ["OnAcquire"] = function(self)
        self:SetValue()
	end,

    ["OnWidthSet"] = function(self, width)
        self:UpdateBar()
    end,

    --- Set the display to percent
    -- @param       state       <bool>
    -- @param       addCount    <bool> 10% <100>
    -- @param       numDecimal  <number>
    ["SetPercent"] = function(self, state, addCount, numDecimal)
        self.isPercent = state
        self.numDecimal = numDecimal
        self.addCount = addCount
        self:UpdateBar()
    end,

    --- Set the value of the bar
    -- @param       minValue            <number>
    -- @param       maxValue            <number>
    -- @param       curValue            <number>
    -- @param       text                <string>
    -- @param       addDisplayFormat    <number> format("%s")
    ["SetValue"] = function(self, minValue, maxValue, curValue, text, addDisplayFormat)
        if curValue and maxValue and curValue > maxValue then
            curValue = maxValue
        end
        minValue, maxValue, curValue = minValue or 0, maxValue or 100, curValue or 0
        self.minValue, self.maxValue, self.curValue = minValue, maxValue, curValue

        curValue = curValue > maxValue and maxValue or curValue

        if self.isPercent then
            local curDisplayValue = (100 / (maxValue-minValue)) * curValue
            if self.addCount then
                self.displayValue = format("%."..(self.numDecimal or 0).."f%% <%.0f>", curDisplayValue, curValue)
            else
                self.displayValue = format("%."..(self.numDecimal or 0).."f%%", curDisplayValue)
            end
        else
            self.displayValue = tostring(curValue)
        end
        self.numberValue = tonumber(self.displayValue)

        local progressWidth = (self.frame:GetWidth() / (maxValue-minValue)) * curValue
        progressWidth = progressWidth > self.frame:GetWidth() and self.frame:GetWidth() or progressWidth
        self.progressBar:SetWidth(progressWidth)

        if text then
            if addDisplayFormat then
                self.text:SetFormattedText(text, self.displayValue)
            else
                self.text:SetText(text)
            end
        else
            self.text:SetText(self.displayValue)
        end

        self.textBg:SetWidth(max(self.text:GetUnboundedStringWidth() + 6, self.minTextWidth))
    end,

    ["UpdateBar"] = function(self)
        if self.curValue then
            self:SetValue(self.minValue, self.maxValue, self.curValue)
        end
    end,

    ["SetBarColor"] = function(self, r, g, b, a)
        self.progressBar:SetColorTexture(r, g, b, a or 0.6)
    end,

    ["SetHeight"] = function(self, height)
        self.height = max(height or 0, self.minHeight)
        self.frame:SetHeight(self.height)
    end,

    ["GetNumberValue"] = function(self)
        return self.numberValue or 0
    end
}

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function Constructor()
    local frame = CreateFrame("FRAME")

    local progressBar = frame:CreateTexture(nil, "BACKGROUND")
    progressBar:SetPoint("TOPLEFT", frame, 0, -1)
    progressBar:SetPoint("BOTTOMLEFT", frame, 0, 1)
    progressBar:SetColorTexture(1, 1, 1, 0.5)

    local gfhFontName, gfhFontHeight, gfhFontFlags = GameFontHighlightSmall:GetFont()
    local textBg = CreateFrame("FRAME", nil, frame) -- GGL.GUI:CreateBoxFrame(frame)
    textBg:SetPoint("CENTER", frame, 0, 0)
    --textBg:SetBackdropBorderColor(1, 1, 1, 0.6)
    textBg:SetHeight(gfhFontHeight + 2)

    local text = textBg:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
    text:SetAllPoints(textBg)
    text:SetJustifyH("CENTER")
    text:SetJustifyV("MIDDLE")
    text:SetText("9999")
    text:SetNonSpaceWrap(false)

    textBg:SetWidth(text:GetUnboundedStringWidth() + 6)

    local widget = {
        type  = Type,
        frame = frame,
        progressBar = progressBar,
        textBg = textBg,
        text = text,
        minWidth = textBg:GetWidth() + 10,
        minHeight = textBg:GetHeight() + 6,
        minTextWidth = text:GetWidth(),
    }

    -- methodes
    for method, func in pairs(methods) do
        widget[method] = func
    end

    widget:SetHeight()

    return AceGUI:RegisterAsWidget(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)
